/*
 * Sumit Khanna - PenguinDreams.org
 *   Free for educational and non-commercial use
 */
import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileReader;
import java.io.InputStreamReader;
import java.io.PrintStream;
import java.security.KeyPair;
import java.security.PublicKey;
import java.security.Security;
import java.security.Signature;
import javax.xml.bind.DatatypeConverter;
import org.bouncycastle.jce.provider.BouncyCastleProvider;
import org.bouncycastle.openssl.PEMReader;


public class JavaVerify {

	  public static PrintStream out = System.out;
	  public static PrintStream err = System.err;
	  
	  public static void main(String[] args) throws Exception {
		  
	    if(args.length < 2 || args.length > 3) {
	      err.println("Usage: java JavaVerify  <pem file> <data file to verify> [signature file]");
	      err.println("\tIf no signature file is given, signature is taken via stdin");
	      System.exit(1);
	    }
	    
	    File pemFile = new File(args[0]);
	    File dataFile = new File(args[1]);
	    
	    if(!dataFile.exists()) {
	    	err.println("Data File Does Not Exist");
	    	System.exit(1);
	    }
	    
	    if(!pemFile.exists()) {
	    	err.println("PEM File Does Not Exist");
	    	System.exit(1);
	    }
	    
	    //BC Provider initalization
	    Security.addProvider(new BouncyCastleProvider());
	    PEMReader pemReader = new PEMReader(new FileReader(pemFile));
	    PublicKey pubKey = ((KeyPair) pemReader.readObject()).getPublic();
	    
	    //load public key
	    Signature sg = Signature.getInstance("SHA1withRSA");
	    sg.initVerify(pubKey);
	    
	    //read data file into signature instance
	    FileInputStream fin = new FileInputStream(dataFile);
	    byte[] data = new byte[(int) dataFile.length()];
	    fin.read(data);
	    sg.update(data);
	    
	    //read signature from file
	    byte[] signature = new byte[0];
	    if(args.length == 3) {
	    	
	    	File sigFile = new File(args[2]);
	    	if(!sigFile.exists()) {
	    		err.println("Signature file could not be found");
	    		System.exit(1);
	    	}
	    	
	    	fin = new FileInputStream(sigFile);
	    	signature = new byte[(int) sigFile.length()];
	    	fin.read(signature);
	    }
	    //read signature from standard in
	    else {
	        BufferedReader in = new BufferedReader(new InputStreamReader(System.in));
	        signature = in.readLine().getBytes();
	    }
	    
	    //validate signature
	    if(sg.verify(DatatypeConverter.parseBase64Binary(new String(signature)))) {
	    	out.println("Signature Verified");
	    	System.exit(0);
	    }
	    else {
	    	out.println("Signature Verification Failed");
	    	System.exit(2);
	    }
	    
	  }
	
}
