#!/usr/bin/env python2

from M2Crypto import EVP, RSA, X509
import sys
import base64
from os import path

# Sumit Khanna - PenguinDreams.org
#   Free for educational and non-commercial use

if __name__ == '__main__':

  if len(sys.argv) != 3:
    sys.stderr.write('Usage ./pysign.py <pem file> <data file to sign>\n')
    sys.exit(1)

  pemFile = sys.argv[1]
  dataFile = sys.argv[2]

  if not path.isfile(dataFile):
    sys.stderr.write('Data file does not exist\n')

  if not path.isfile(pemFile):
    sys.stderr.write('PEM file does not exist. Generating\n')

    #keysize in bits is 2048, RSA public exponent is 65537
    # Callback supresses ....++ output on key generation
    key = RSA.gen_key(2048, 65537,callback=lambda x, y, z:None) 

    #Using a cipher of None prevents being prompted for a passphrase
    # A callback function can also be supplied
    key.save_pem(pemFile, cipher=None)


  key = EVP.load_key(pemFile)
  key.reset_context(md='sha1')
  key.sign_init()
  key.sign_update(open(dataFile,'r').read())

  #Signatures are binary, so we base64 encode the result for portability
  print(base64.b64encode(key.sign_final()))

